from pydantic import BaseModel

import pys


# An author
@pys.saveable
class Author(BaseModel):
    name: str


# And a book
@pys.saveable
class Book(BaseModel):
    title: str


def test_sample():
    storage = pys.storage('.storage')

    # A few books of Leo Tolstoy
    leo = Author(name='Leo Tolstoy')
    war_and_peace = Book(title='War and peace')
    for_kids = Book(title='For Kids')

    storage.save(leo)
    storage.save(war_and_peace, leo)
    storage.save(for_kids, leo)

    leo_books = list(storage.list(Book, leo))
    assert len(leo_books) == 2
    assert war_and_peace in leo_books
    assert for_kids in leo_books



import uuid
from typing import Any

from pydantic import BaseModel


class ModelWithID(BaseModel):
    """
    Base class for models with `id` field prefilled with random UUID if not initialized.
    """
    id: str = None

    def __init__(self, **data: Any) -> None:
        """
        Initialize a model
        :param data: model fields
        """
        super().__init__(**data)
        if self.id is None:
            self.id = str(uuid.uuid4())

    def __my_id__(self):
        return self.id

    def __json__(self):
        return self.model_dump_json()



from pydantic import BaseModel

import pys


def test_sample_pydantic2():
    # An author
    @pys.saveable
    class Author(BaseModel):
        name: str

    # And a book
    @pys.saveable
    class Book(BaseModel):
        title: str

    storage = pys.storage('.storage')

    # A few books of Leo Tolstoy
    leo = Author(name='Leo Tolstoy')
    war_and_peace = Book(title='War and peace')

    # Save Leo's book
    storage.save(leo)
    wnp_id = storage.save(war_and_peace, leo)

    # One more author :)
    gpt = Author(name='Chat GPT')

    # Do we have the same book by GPT?
    gpt_war_and_peace = storage.load(Book, wnp_id, gpt)
    assert gpt_war_and_peace is None

    # Now it has :)
    storage.save(war_and_peace, gpt)
    gpt_war_and_peace = storage.load(Book, wnp_id, gpt)
    assert gpt_war_and_peace is not None



from dataclasses import dataclass

import pys


@pys.saveable(field_as_id='some_other')
@dataclass
class A:
    some_other: str


@pys.saveable
@dataclass
class B:
    id: str


@pys.saveable
@dataclass
class C:
    name: str


def test_with_id():
    a = A(some_other='xyz')
    assert a.__my_id__() is not None
    assert a.some_other is not None
    assert a.some_other == a.__my_id__()
    assert a.__json__() is not None
    assert a.__json__() == '{"some_other":"' f'{a.__my_id__()}' '"}'

    b = B(id='123')
    assert b.__my_id__() is not None
    assert b.id == b.__my_id__()
    assert b.id == '123'
    assert b.__json__() is not None
    assert b.__json__() == '{"id":"' f'{b.id}' '"}'

    c = C(name='C class')
    assert c.__my_id__() is not None
    assert c.__json__() is not None
    assert c.__json__() == f'{{"name":"{c.name}"}}'
